/***********************************************************************************
 * LabTestSim - Laboratory Test Simulator using Elastoplastic models               *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>     *
 *                                                                                 *
 * This file is part of LabTestSim.                                                *
 *                                                                                 *
 * LabTestSim is free software; you can redistribute it and/or modify it under the *
 * terms of the GNU General Public License as published by the Free Software       *
 * Foundation; either version 2 of the License, or (at your option) any later      *
 * version.                                                                        *
 *                                                                                 *
 * LabTestSim is distributed in the hope that it will be useful, but WITHOUT ANY   *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.        *
 *                                                                                 *
 * You should have received a copy of the GNU General Public License along with    *
 * LabTestSim; if not, write to the Free Software Foundation, Inc., 51 Franklin    *
 * Street, Fifth Floor, Boston, MA 02110-1301, USA                                 *
 ***********************************************************************************/

#if HAVE_CONFIG_H
# include <config.h>
#else
# define PACKAGE_STRING "LabTestSim version ?.?"
#endif

#include <stdexcept>
#include <iostream>
#include <fstream>
#include <string>

#include "inputdata.h"
#include "labtestsim.h"
#include "readmat.h"

// external
#include "mechsys/parameters/parameters.h"
#include "mechsys/models/subcam.h"
#include "mechsys/models/subtij.h"
#include "mechsys/base/mechsys.h"
#if WITHGUI
# include "gui/gui.h"
#endif

using namespace std;

bool CheckInputConsole(int argc, char **argv, InputData & ID);
bool RunSim(InputData & ID);
bool GenPltFiles(InputData & ID);

bool labtestsim_main(int argc, char **argv) // return FALSE if had problems
{
	bool IsConsole = true;
	if (argc==2)
		if (strcmp(argv[1],"--gui")==0) IsConsole = false;

	if (IsConsole)
	{
		// InputData
		InputData ID;

		// Check input and set filenames
		if (!CheckInputConsole(argc, argv, ID)) return false;
		ID.SetFilenames();

		// Call RunSim
		if (!RunSim(ID)) return false;

		// plt Output
		if (!GenPltFiles(ID)) return false;
	}
	else
	{
#if WITHGUI
		Gui gui(argc, argv, &RunSim, &GenPltFiles);
#else
		cout << "GUI not available ... :(\n";
#endif
	}
	return true; // OK
}

int main(int argc, char **argv)
{
	try
	{
		if (!labtestsim_main(argc,argv)) return 1;
	}
	catch(exception & e)
	{
		cout << "Exception: " << e.what() << endl;
		return 1;
	}
	return 0; // OK
}

bool CheckInputConsole(int argc, char **argv, InputData & ID) // return FALSE if had problems
{

	cout << endl << PACKAGE_STRING << ", Copyright (C) 2005 Dorival M. Pedroso\n";
	cout << "    LabTestSim comes with ABSOLUTELY NO WARRANTY.\n";
	cout << "    This is free software, and you are welcome\n";
	cout << "    to redistribute it under certain conditions.\n";
	cout << endl;
	cout << "How to plot (with gnuplot), Ex.:\n";
	cout << " $ cat FO1.CTP.01.subcam.plt | gnuplot\n";
	cout << " or\n";
	cout << " $ gnuplot\n";
	cout << " gnuplot> load 'FO1.CTO.01.subcam.plt'\n";
	cout << endl;

	if (argc!=9)
	{
		cout << "You must enter: " << argv[0] << " DATDIR MAT TST NUM MDL nDiv nOut maxEd\n";
		cout << "In which:\n";
		cout << "\t DATDIR = Directory with xmaterials.dat and test's trajectories\n"; // args[1]
		cout << "\t    MAT = Material's key {FO1,FO2,FO4,FO8, FNW,FCH, TSD,PSD}\n";    // args[2]
		cout << "\t    TST = Test's key {CTX,CTP,CTA,CTR, TTX,TTP, HCY,PSA}\n";        // args[3]
		cout << "\t    NUM = Number of the test {01...NN}\n";                          // args[4]
		cout << "\t    MDL = Constitutive model {subcam,subtij}\n";                    // args[5]
		cout << "\t   nDiv = Number of divisions for each path (nDiv>=1)\n";           // args[6]
		cout << "\t   nOut = Number of output for each path (1<=nOut<=nDiv)\n";        // args[7]
		cout << "\t  maxEd = Max Value for deviatoric strain ed (%) (maxEd>0%)\n";     // args[8]
		cout << "EXAMPLE:\n";
		cout << "         " << argv[0] << " ../data FO1 CTP 01 subcam 100 10 15\n";
		cout << "         " << argv[0] << " ../data FO1 CTP 01 subtij 100 10 15\n";
		cout << "or perhaps you want a Graphical User Interface - GUI, if so, please type:\n";
		cout << "         " << argv[0] << " --gui\n";
		return false;
	}

	ID.DATDIR =      argv[1];
	   ID.MAT =      argv[2];
	   ID.TST =      argv[3];
	   ID.NUM =      argv[4];
	   ID.MDL =      argv[5];
	  ID.nDiv = atoi(argv[6]);
	  ID.nOut = atoi(argv[7]);
	 ID.maxEd = atof(argv[8]);

	if (ID.nDiv<1) 
	{
		cout << "ERROR: nDiv=" << ID.nDiv << " must be greater (or equal) than (to) 1 !!!\n";
		return false;
	}
	if (ID.nOut<1) 
	{
		cout << "ERROR: nOut=" << ID.nOut << " must be greater (or equal) than (to) 1 !!!\n";
		return false;
	}
	if (ID.nOut>ID.nDiv) 
	{
		cout << "ERROR: nOut=" << ID.nOut << " must be lesser (or equal) than (to) nDiv=" << ID.nDiv << " !!!\n";
		return false;
	}
	if (ID.maxEd<=0.0) 
	{
		cout << "ERROR: maxEd=" << ID.maxEd << " must be greater than 0.0 !!!\n";
		return false;
	}

	return true;
	
} // CheckInputConsole()

bool RunSim(InputData & ID) // return FALSE if had problems
{
	// Read Materials information
	ReadMat RM(ID.XMatFN);

	// Parameters
	MechSys::Parameters Prms;
	string              strMatKey(ID.MAT);
	if (!RM.SetPrms(&Prms,strMatKey))
	{
		cout << "ERROR: Could not read xmaterials.dat and hence set parameters !!!\n";
		cout << RM.GetErrors();
		return false;
	}

	// Models
	MechSys::SubCam SubCamMDL(MechSys::Mod); //subcam_model.SetSimpleOut(false);
	MechSys::SubTij SubTijMDL;               //subtij_model.SetSimpleOut(false);
	
	// Simulator
	LabTestSim Sim;
	
	// Select Model and set LabSimulator
	switch (MechSys::GetModelID(ID.MDL.c_str()))
	{
	case MechSys::subcam: // SubCam
		{
			if (!Sim.Set(ID.TST.c_str(),&Prms,&SubCamMDL,ID.PatFN,ID.nDiv,ID.nOut,ID.maxEd))
			{
				cout << "ERROR: Could not set Simulator !!!\n";
				cout << Sim.GetErrors();
				return false;
			}
			break;
		}
	case MechSys::subtij: // SubTij
		{
			if (!Sim.Set(ID.TST.c_str(),&Prms,&SubTijMDL,ID.PatFN,ID.nDiv,ID.nOut,ID.maxEd))
			{
				cout << "ERROR: Could not set Simulator !!!\n";
				cout << Sim.GetErrors();
				return false;
			}
			break;
		}
	default:
		cout << "ERROR: Invalid Model !!!\n";
		return false;
	}

	// Run Test
	if (!Sim.RunTest())
	{
		cout << "ERROR: Could not RunTest() !!!\n";
		cout << Sim.GetErrors();
		return false;
	}

	// nfo Output
	ofstream nfoFile(ID.nfoFN.c_str(),std::ios::out);
	if (nfoFile)
	{
		nfoFile << "Parameters:\n" << endl;
		nfoFile << Prms << endl;
		nfoFile.close();
		cout << "*** File <" << ID.nfoFN << "> saved ***\n";
	}
	else
	{
		cout << "ERROR: Could not open file <" << ID.nfoFN << "> !!!\n";
		return false;
	}

	// cal Output
	ofstream calFile(ID.calFN.c_str(),std::ios::out);
	if (calFile)
	{
		calFile << Sim;
		calFile.close();
		cout << "*** File <" << ID.calFN << "> saved ***\n";
	}
	else
	{
		cout << "ERROR: Could not open file <" << ID.calFN << "> !!!\n";
		return false;
	}

	return true; // OK

} // RunSim

bool GenPltFiles(InputData & ID)
{
	ofstream file(ID.pltFN.c_str(),std::ios::out);
	if (file)
	{
		// plot with: $ cat pltFN | gnuplot
		// q x ed
		file << "# Lines: (lt)"                                        << std::endl;
		file << "# 1 red                      7 yellow"                << std::endl;
		file << "# 2 green                    8 orange"                << std::endl;
		file << "# 3 blue                     9 red"                   << std::endl;
		file << "# 4 magenta                 10 green"                 << std::endl;
		file << "# 5 cyan                    11 blue"                  << std::endl;
		file << "# 6 brown                   12 magenta"               << std::endl;
		file << "#                           13 cyan"                  << std::endl;
		file << ""                                                     << std::endl;
		file << "# Points: (pt)"                                       << std::endl;
		file << "# 1 +                        7 (f) circle"            << std::endl;
		file << "# 2 cross                    8 (o) triangle"          << std::endl;
		file << "# 3 *                        9 (f) triangle"          << std::endl;
		file << "# 4 (o) square              10 (o) inverse-triangle"  << std::endl;
		file << "# 5 (f) square              11 (f) inverse-triangle"  << std::endl;
		file << "# 6 (o) circle              12 (o) diamond"           << std::endl;
		file << "#                           13 (f) diamond"           << std::endl;
		file << "# o: open"                                            << std::endl;
		file << "# f: filled"                                          << std::endl;
		file << ""                                                     << std::endl;
		file << "# --------------------------------------------------" << std::endl;
		file << ""                                                     << std::endl;
		file << "set line style 1 lt 1 pt 1"                           << std::endl;
		file << "set line style 2 lt 2 pt 2"                           << std::endl;
		file << "set line style 3 lt 3 pt 6"                           << std::endl;
		file << "set line style 4 lt 3 pt 5"                           << std::endl;
		file << ""                                                     << std::endl;
		file << "set term postscript eps enhanced color solid"         << std::endl;
		file << "set output '" << ID.epsFN                             << std::endl;
		file << ""                                                     << std::endl;
		file << "set xlabel '{/Symbol=20 e}{_D (%)}'"                  << std::endl;
		file << "set ylabel 'q/p'"                                     << std::endl;
		file << ""                                                     << std::endl;
		//file << "set tmargin 7"                                        << std::endl;
		//file << "set bmargin 7"                                        << std::endl;
		//file << "set lmargin 20"                                       << std::endl;
		//file << "set rmargin 20"                                       << std::endl;
		file << "set grid"                                             << std::endl;
		file << "set key right bottom"                                 << std::endl;
		file << "set key box"                                          << std::endl;
		file << ""                                                     << std::endl;
		file << "plot '" << ID.DatFN << "' u ($10):($7/$6) w p title 'Data', \\"        << std::endl;
		file << " '" << ID.calFN << "' u ($13*$12):($14*$10/$9) w l title 'Simulation'" << std::endl;
		file.close();
		cout << "*** File <" << ID.pltFN << "> saved ***\n";
		// generate eps file
		std::string command("cat ");
		command.append(ID.pltFN);
		command.append(" | gnuplot");
		std::cout << "command> " << command << std::endl;
		int flag = std::system(command.c_str());
		if (flag!=0)
		{
			std::cout << "ERROR: command failed\n";
			return false;
		}
		// generate png file
		command = "eps2png ";
		command.append(ID.epsFN);
		std::cout << "command> " << command << std::endl;
		flag = std::system(command.c_str());
		if (flag!=0)
		{
			std::cout << "ERROR: command failed\n";
			return false;
		}
	}
	else
	{
		cout << "ERROR: Could not open file <" << ID.pltFN << "> !!!\n";
		return false;
	}

	return true; // OK
}
