/*************************************************************************************
 * MechSys - A C++ library to simulate (Continuum) Mechanical Systems                *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of MechSys.                                                     *
 *                                                                                   *
 * MechSys is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

// OBS.:
//   Internally, the values are stored as Col-Major, thus a pointer
//   can be passed to FORTRAN routines very easily.

#ifndef LINALG_MATRIX_H
#define LINALG_MATRIX_H

#include <iostream>
#include <iomanip>
#include <cassert>

namespace LinAlg
{

class Matrix
{
public:
	// Constructor
	Matrix(int Rows, int Cols) : _rows(Rows), _cols(Cols)
	{
		assert(_rows>0);
		assert(_cols>0);
		_values = new double [_rows*_cols];
		for (int i=0; i<_rows*_cols; ++i)
			_values[i] = 0.0;
	}
	// Destructor
	~Matrix()
	{
		delete [] _values;
	}
	// Get
	         int   Rows()   const { return _rows; }
	         int   Cols()   const { return _cols; }
	      double * GetPtr()       { return _values; }
	const double * GetPtr() const { return _values; }
	// Set
	void SetValues(double Value)
	{
		for (int i=0; i<_rows*_cols; ++i)
			_values[i] = Value; 
	}
	// operators
	void operator= (const Matrix & R)
	{
		assert(&R!=this);
		assert(R.Rows()==_rows);
		assert(R.Cols()==_cols);
		for (int i=0; i<_rows*_cols; ++i)
			_values[i] = R._values[i];
	}
	double & operator() (int i, int j)
	{
		assert(i>=0 & i<_rows);
		assert(j>=0 & j<_rows);
		return _values[j*_rows+i]; // Col major
	}
	const double & operator() (int i, int j) const
	{
		assert(i>=0 & i<_rows);
		assert(j>=0 & j<_rows);
		return _values[j*_rows+i]; // Col major
	}
private:
	int      _rows;
	int      _cols;
	double * _values;
}; // class Matrix

}; // namespace LinAlg

std::ostream & operator<< (std::ostream & os, const LinAlg::Matrix & M);
std::istream & operator>> (std::istream & is, LinAlg::Matrix & M);

#endif
