/*************************************************************************************
 * MechSys - A C++ library to simulate (Continuum) Mechanical Systems                *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of MechSys.                                                     *
 *                                                                                   *
 * MechSys is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

#include <sstream>
#include "intvars.h"
#include "genericep.h"

namespace MechSys
{

// Inherited methods

bool GenericEP::Set(Parameters const * Prms, const Tensr2 & Sig)
{
	_prms = Prms;
	 _sig = Sig;
	   _D = _IIsym / (1.0+_prms->nu()) + _IdyI * (_prms->nu()/((1.0+_prms->nu())*(1.0-2.0*_prms->nu())));
	   _C = _IIsym * (1.0+_prms->nu()) - _IdyI *  _prms->nu();
	  _ce = (1.0-2.0*_prms->nu())*(1.0+_prms->vri())/_prms->kap();
	if (!_model_specific_set()) return false;
	if (!_start_intvars(_sig))  return false;
	if (!_update_state())       return false;
	return true;
}

bool GenericEP::Reset(const Tensr2 & Sig)
{
	_sig = Sig;
	if (!_model_specific_set()) return false;
	if (!_start_intvars(_sig))  return false;
	if (!_update_state())       return false;
	return true;
}

bool GenericEP::Actualize(const Tensr2 & Dsig, Tensr2 & Deps) // return Deps
{
	Tensr4 _Ce = _C / _E;
	double  dL = (_v * Dsig) / _cp;
	      _sig = _sig + Dsig;
	if (dL<0) // Elastic
	{
		Deps = _Ce * Dsig;
		if (!_force_zalpha(_sig)) return false;
	}
	else      // Elastoplastic
	{
		Deps = _Ce * Dsig + _r * dL;
		_z.Update(_HHa,_HHb,dL);
	}
	if (!_update_state()) return false;
	return true;
}

bool GenericEP::StressUpdate(const Tensr2 & Deps, Tensr2 & Dsig) // return Dsig
{
	if (!_is_scheme_set)                        return false;
	// Verificar situacao EL ou EP
	Tensr2 DeDeps = _De * Deps;
	double  numDL = _v * DeDeps; // numerator of DeltaLambda
	if (numDL<0.0) // elastic
	{
		Dsig = DeDeps;
		_sig = _sig + Dsig;
		if (!_force_zalpha(_sig)) return false;
		if (!_update_state())     return false;
	}
	else // elastoplastic
		if (!_scheme->StressUpdate(this,Deps,Dsig)) return false;
	return true;
}

Tensr4 * GenericEP::TgCompliance() // C
{
	Tensr4 _Ce = _C / _E;
	      _Cep = _Ce + (_r & _v) / _cp;
	return (&_Cep);
}

Tensr4 * GenericEP::TgStiffness() // D
{
	Tensr2 A = _De * _r;
	    _Dep = _De - (A & _b);
	return (&_Dep);
}

void GenericEP::Out(std::ostringstream & oss) const
{
	oss << "------------------------------------------------------------------------------\n";
	oss << "                    Generic Elastoplastic Model - ";
	if (GetID()==subcam) oss << "SubCam\n";
	if (GetID()==subtij) oss << "SubTij\n";
	oss << "------------------------------------------------------------------------------\n";
	oss << "Sig = \n" << _sig << std::endl;
	oss << "  z = "   << _z   << std::endl;
	oss << "  E = "   << _E   << std::endl;
	if (!_is_simple_out)
	{
		oss << " De =\n"  << _De  << std::endl;
		oss << "  r = "   << _r   << std::endl;
		oss << "  v = "   << _v   << std::endl;
		oss << " cp = "   << _cp  << std::endl;
		oss << "  b = "   << _b   << std::endl;
		oss << " ya = "; for (int i=0; i<_z.nIVLS(); ++i)
		oss << std::setw(12) << std::showpos << std::fixed << std::setprecision(6) << _ya[i];
		oss << std::endl;
		oss << "HHa = "; for (int i=0; i<_z.nIVLS(); ++i)
		oss << std::setw(12) << std::showpos << std::fixed << std::setprecision(6) << _HHa[i];
		oss << std::endl;
		oss << "HHb = "; for (int i=0; i<_z.nIVns(); ++i)
		oss << std::setw(12) << std::showpos << std::fixed << std::setprecision(6) << _HHb[i];
		oss << std::endl;
	}
	_model_specific_out(oss);
	oss << "------------------------------------------------------------------------------\n";
}

// Private functions

bool GenericEP::_update_state()
{
	        _E = _ce*(_sig[0]+_sig[1]+_sig[2]);
	       _De = _D * _E;
	if (!_model_specific_update()) return false;
	if (!_calculate_gradients())   return false; // calc: r v ya
	if (!_calculate_hardening())   return false; // calc: HHa HHb
	       _cp = 0.0; for (int i=0; i<_z.nIVLS(); ++i)
	       _cp = _cp - _ya[i] * _HHa[i];
	Tensr2   A = _De * _r;
	double Phi = _v * A + _cp;
	        _b = (_v * _De) / Phi; // (dL = b:dEps)
	return true;
}

void GenericEP::_dstress_given_dstrain(const Tensr2 & Deps, Tensr2 & Dsig, IntVars & Dz)
{
	double  dL = _b * Deps;
	Tensr2 dee = Deps - _r * dL; // deelastic
	      Dsig = _De * dee;
	        Dz.Set(_HHa,_HHb,dL);
}

}; // namespace MechSys
