/*************************************************************************************
 * MechSys - A C++ library to simulate (Continuum) Mechanical Systems                *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of MechSys.                                                     *
 *                                                                                   *
 * MechSys is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

#include <iostream>
#include "nlelastic.h"

namespace MechSys
{

bool NlElastic::Set(Parameters const * Prms, const Tensr2 & Sig)
{
	_prms = Prms;
	 _sig = Sig;
	   _D = _IIsym / (1.0+_prms->nu()) + _IdyI * (_prms->nu()/((1.0+_prms->nu())*(1.0-2.0*_prms->nu())));
	   _C = _IIsym * (1.0+_prms->nu()) - _IdyI *  _prms->nu();
	  _ce = (1.0-2.0*_prms->nu())*(1.0+_prms->vri())/_prms->kap();

	if (!_update_state()) return false;
	return true;
}

bool NlElastic::Reset(const Tensr2 & Sig)
{
	_sig = Sig;
	if (!_update_state()) return false;
	return true;
}

bool NlElastic::Actualize(const Tensr2 & Dsig, Tensr2 & Deps) // return Deps
{
	 _Ce = _C / _E;
	Deps = _Ce * Dsig;
	return true;
}

bool NlElastic::StressUpdate(const Tensr2 & Deps, Tensr2 & Dsig) // return Dsig
{
	if (!_is_scheme_set)                        return false;
	if (!_scheme->StressUpdate(this,Deps,Dsig)) return false;
	return true;
}

Tensr4 * NlElastic::TgCompliance () // C
{
	_Ce = _C / _E;
	return (&_Ce);
}

Tensr4 * NlElastic::TgStiffness  () // D
{
	_De = _D * _E;
	return (&_De);
}

void NlElastic::Out(std::ostringstream & oss) const
{
	oss << "------------------------------------------------------------------------------\n";
	oss << "                          Non-Linear Elastic Model\n";
	oss << "------------------------------------------------------------------------------\n";
	oss << "Sig = \n" << _sig << std::endl;
	oss << "p = " << _p   << std::endl;
	oss << "E = " << _E   << std::endl;
	oss << "------------------------------------------------------------------------------\n";
}

bool NlElastic::_update_state()
{
	_p = (_sig[0]+_sig[1]+_sig[2])/3.0;
	_E = 3.0*_ce*_p;
	return true;
}

}; // namespace MechSys
