/*************************************************************************************
 * MechSys - A C++ library to simulate (Continuum) Mechanical Systems                *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of MechSys.                                                     *
 *                                                                                   *
 * MechSys is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

#include <cmath>
#include <sstream>
#include "../invariants/stressinvs.h"
#include "subcam.h"

namespace MechSys
{

SubCam::SubCam(SubCamSurfType SurfType) : GenericEP(1,1), _surf_type(SurfType) {}

bool SubCam::_model_specific_set()
{
	_chi = (_prms->lam() - _prms->kap()) / (1.0 + _prms->vri());
	return true; 
}

bool SubCam::_force_zalpha(Tensr2 & Sig)
{
	StressInvs si(Sig);
	double p;
	double q;
	if (!si.Get_p_q(p,q)) return false;
	switch (_surf_type)
	{
	case Ori:
	case Mod:
		_z[0] = p + pow(q/_prms->M(),2.0) / p;
		break;
	}
	return true; 
}

bool SubCam::_start_intvars(Tensr2 & Sig)
{
	if (!_force_zalpha(Sig)) return false;
	_z[1] = _z[0] * _prms->d2();
	return true; 
}

bool SubCam::_model_specific_update()
{
	StressInvs si(_sig);
	if (!si.Get_p_q(_p,_q)) return false;
	return true; 
}

bool SubCam::_calculate_gradients()
{
	switch (_surf_type)
	{
	case Ori:
	case Mod:
		{
			double MM = pow(_prms->M(),2.0);
			Tensr2 dev_sig = _sig - (_I * _p);
			double k = MM*(2.0*_p-_z[0])/3.0;
			      _r = _I*k + dev_sig*3.0;
			      _v = _r;
			  _ya[0] = -MM*_p;
			break;
		}
	}
	return true; 
}

bool SubCam::_calculate_hardening()
{
	     _G = _prms->c2()*pow((_prms->lam()-_prms->kap())*log(_z[1]/_z[0]),2.0);
	  _tr_r = _r[0]+_r[1]+_r[2];
	_HHa[0] = _z[0]*(_tr_r+_G/_p)/_chi;
	_HHb[0] = _z[1]*_tr_r/_chi;
	return true; 
}

void SubCam::_model_specific_out(std::ostringstream & oss) const
{
	oss << "chi = "   << _chi << std::endl;
	oss << "  p = "   << _p   << std::endl;
	oss << "  q = "   << _q   << std::endl;
	oss << "  G = "   << _G   << std::endl;
	//if (!_is_simple_out)
	//{
	//}
}

/*
double SubCam::_calculate_f() const
{
	double f;
	switch (_surf_type)
	{
	case Ori:
	case Mod:
		f = pow(_prms->M(),2.0)*_p*(_p-_z[0])+pow(_q,2.0);
		break;
	}
	return f;
}
*/

}; // namespace MechSys
