/*************************************************************************************
 * MechSys - A C++ library to simulate (Continuum) Mechanical Systems                *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of MechSys.                                                     *
 *                                                                                   *
 * MechSys is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

#include <cmath>
#include <string>
#include <sstream>
#include "../invariants/stressinvs.h"
#include "tijtensor.h"
#include "subtij.h"

namespace MechSys
{

SubTij::SubTij() : GenericEP(1,1), _t(SubTij::TSMIN) {} // nIVLS=1, nIVns=1

bool SubTij::_model_specific_set()
{
	double DELT = _sig[0]*0.00002; // to add to _sig 

	// Avoid Hydrostatic state
	double d0 = fabs(_sig[0] - _sig[1]); 
	double d1 = fabs(_sig[1] - _sig[2]);
	double d2 = fabs(_sig[2] - _sig[0]); 
	double d3 = fabs(_sig[3])+fabs(_sig[4])+fabs(_sig[5]);
	if (d0<=HYDDIFFMIN & d1<=HYDDIFFMIN & d2<=HYDDIFFMIN & d3<=HYDDIFFMIN)
	{
		/*
		std::cout << "Hydrostatic state ..........\n";
		std::cout << "_sig = " << _sig << std::endl;
		*/
		_sig[1] = _sig[1] + DELT;
	}

	_chi = (_prms->lam() - _prms->kap()) / (1.0 + _prms->vri());
	return true; 
}

bool SubTij::_force_zalpha(Tensr2 & Sig)
{
	StressInvs si(Sig);
	double tn;
	double ts;
	double rz;
	if (!si.Get_tn_ts_rz(tn,ts,rz))
	{
		_errors.append(si.GetErrors());
		_errors.append("SubTij::_force_zalpha:: Get_tn_ts_rz(_tn, _ts, _rz) failed !\n"); 
		return false;
	}
	_z[0] = tn*exp(pow(ts/(_prms->mus()*tn),_prms->bet())/_prms->bet()); // tn1
	return true;
}

bool SubTij::_start_intvars(Tensr2 & Sig)
{
	if (!_force_zalpha(Sig)) return false;
	_z[1] = _z[0] * _prms->d1();
	return true;
}

bool SubTij::_model_specific_update()
{
	if (!_t.Set(_sig))
	{
		_errors.append(_t.GetErrors());
		return false;
	}
	_phi = pow(_t.ts()/(_prms->mus()*_t.tn()),_prms->bet());
	return true; 
}

bool SubTij::_calculate_gradients()
{
	if (!_t.CalcDerivs())
	{
		_errors.append(_t.GetErrors());
		return false;
	}
	_df_dtn = (1.0-_phi)/_t.tn();
	_df_dts = _phi/_t.ts();
	     _r =    (*_t.aij())*_df_dtn + (*_t.dts_dt())*_df_dts; // df_dt
	     _v = (*_t.dtn_ds())*_df_dtn + (*_t.dts_ds())*_df_dts; // df_ds
	 _ya[0] = -1.0/_z[0];
	return true; 
}

bool SubTij::_calculate_hardening()
{
	     _G = _prms->c1()*pow((_prms->lam()-_prms->kap())*log(_z[1]/_z[0]),2.0);
	  _tr_r = _r[0]+_r[1]+_r[2];
	_HHa[0] = _z[0]*(_tr_r+_G/_t.tn())/_chi;
	_HHb[0] = _z[1]*_tr_r/_chi;
	return true; 
}

void SubTij::_model_specific_out(std::ostringstream & oss) const
{
	oss << "chi = "   << _chi    << std::endl;
	oss << " tn = "   << _t.tn() << std::endl;
	oss << " ts = "   << _t.ts() << std::endl;
	oss << "  G = "   << _G      << std::endl;
	if (!_is_simple_out)
	{
		oss << "   aij = " << (*_t.aij())    << std::endl;
		oss << "   tij = " << (*_t.tij())    << std::endl;
		oss << "dts_dt = " << (*_t.dts_dt()) << std::endl;
		oss << "dtn_ds = " << (*_t.dtn_ds()) << std::endl;
		oss << "dts_ds = " << (*_t.dts_ds()) << std::endl;
		oss << "df_dtn = " << _df_dtn        << std::endl;
		oss << "df_dts = " << _df_dts        << std::endl;
	}
	if (_errors.size()!=0)
		oss << "ERRORS: \n" << _errors << std::endl;
}

/*
double SubTij::_calculate_f() const
{
	double f;
	f = log(_t.tn()/_z[0])+pow(_t.ts()/(_prms->mus()*_t.tn()),_prms->bet())/_prms->bet();
	return f;
}
*/

}; // namespace MechSys
