/*************************************************************************************
 * MechSys - A C++ library to simulate (Continuum) Mechanical Systems                *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of MechSys.                                                     *
 *                                                                                   *
 * MechSys is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

#include <cmath>
#include <cassert>
#include "../base/mechsys.h"
#include "../invariants/charinvs.h"
#include "../invariants/stressinvs.h"
#include "tijtensor.h"

MechSys::TijTensor::TijTensor(double TsMin)
		: _ts_min(TsMin),
		  _is_stress_ok(false),
		  _is_derivs_ok(false)
{}

bool MechSys::TijTensor::Set(const MechSys::Tensr2 & Sig)
{
	_errors.clear();
	
	_sig = Sig;
	
	MechSys::CharInvs ci(_sig);
	_I1 = ci.I(0);
	_I2 = ci.I(1);
	_I3 = ci.I(2);
	if (fabs(_I2)<=0.0)
	{
		_errors.append("TijTensor::Set:: fabs(_I2)<=0.0 failed !\n");
		return false;
	}

	MechSys::StressInvs si(_sig);
	if (!si.Get_tn_ts_rz(_tn, _ts, _rz))
	{
		_errors.append(si.GetErrors());
		_errors.append("TijTensor::Set:: Get_tn_ts_rz(_tn, _ts, _rz) failed !\n"); 
		return false;
	}
	if (!MechSys::tensor_sqrt(_sig, _tau))
	{
		_errors.append("TijTensor::Set:: tensor_sqrt(_sig, _tau) failed !\n"); 
		return false;
	}
	if (!MechSys::tensor_inv(_tau, _inv_tau))
	{
		_errors.append("TijTensor::Set:: tensor_inv(_tau, _inv_tau) failed !\n"); 
		return false;
	}

	_aij = _inv_tau * _rz;
	MechSys::tensor_mult(_aij, _sig, _tij);

	_is_stress_ok = true;
	_is_derivs_ok = false;
	return true;
}

bool MechSys::TijTensor::CalcDerivs()
{
	if (!_is_stress_ok)
	{
		_errors.append("TijTensor::CalcDerivs:: _is_stress_ok = false !\n"); 
		return false;
	}
	if (fabs(_ts)<_ts_min)
	{
		_errors.append("TijTensor::CalcDerivs:: fabs(_ts)<_ts_min failed !\n"); 
		return false;
	}

	Tensr2 sigsig;
	MechSys::tensor_mult(_sig,_sig,sigsig);
	
	_dI0_ds[0]=1.0; _dI0_ds[1]=1.0; _dI0_ds[2]=1.0; // = I
	_dI0_ds[3]=0.0; _dI0_ds[4]=0.0; _dI0_ds[5]=0.0;
	_dI1_ds = _dI0_ds*_I1 - _sig;
	_dI2_ds = sigsig - _sig*_I1 + _dI0_ds*_I2;
	_dtn_ds = _dI2_ds*(3.0/_I2) - _dI1_ds*(_tn/_I2);
	
	double alp = _tn/(6.0*_ts);
	double bet = _tn*(6.0*_tn-_I1)/(6.0*_ts*_I2);
	double gam = (_I1-6.0*_tn)/(2.0*_ts*_I2);
	
	_dts_ds = _dI0_ds*alp + _dI1_ds*bet + _dI2_ds*gam;
	_dts_dt = (_tij - _aij*_tn)/_ts;
		
	_is_derivs_ok = true;
	return true;
}
