/*************************************************************************************
 * MechSys - A C++ library to simulate (Continuum) Mechanical Systems                *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of MechSys.                                                     *
 *                                                                                   *
 * MechSys is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

#ifndef MECHSYS_PARAMETERS_H
#define MECHSYS_PARAMETERS_H

#include <iostream>
#include <cassert>
#include "../base/mechsys.h"

namespace MechSys { class Parameters; }
std::ostream & ::operator<< (std::ostream & os, const MechSys::Parameters & mp);

namespace MechSys
{

class Parameters
{

	friend std::ostream & ::operator<< (std::ostream & os, const MechSys::Parameters & mp);
	
public:

	Parameters();
	void Calculatevri(MechSys::Tensr2 & Sig);
	void CalculateM();
	void CalculateMus();

	void Setlam(double lam) { _lam=lam; }
	void Setkap(double kap) { _kap=kap; }
	void Setnu (double nu)  {  _nu=nu;  }
	void Setrcs(double rcs) { _rcs=rcs; _calculate_phics(); }
	void Setenc(double enc) { _enc=enc; }
	void Setpnc(double pnc) { _pnc=pnc; }
	void Setbet(double bet) { _bet=bet; }
	void Setc1 (double c1)  {  _c1=c1;  }  // sub-tij
	void Setc2 (double c2)  {  _c2=c2;  }  // sub-cam
	void Setd1 (double d1)  {  _d1=d1;  }  // sub-tij
	void Setd2 (double d2)  {  _d2=d2;  }  // sub-cam

	double lam() const { assert(_lam>0.0);     return _lam; }
	double kap() const { assert(_kap>0.0);     return _kap; }
	double  nu() const { assert( _nu>=0.0);    return  _nu; }
	double vri() const { assert(_is_vri_calc); return _vri; }
	double bet() const { assert(_bet>0.0);     return _bet; }
	double  c1() const { assert( _c1>0.0);     return  _c1; }
	double  c2() const { assert( _c2>0.0);     return  _c2; }
	double  d1() const { assert( _d1>0.0);     return  _d1; }
	double  d2() const { assert( _d2>0.0);     return  _d2; }
	double   M() const { assert(_is_M_calc);   return   _M; }
	double mus() const { assert(_is_mus_calc); return _mus; }

private:

	// basic
	double _lam;
	double _kap;
	double _nu;  // Poisson

	// initial condition
	double _enc; //
	double _pnc; // 
	double _vri; // void ratio (initial)
	bool   _is_vri_calc;

	// strength
	double _rcs;   // (sig1/sig3) at critical state (compression)
	double _phics; // shear angle at compression (degrees)
	double _M;     // CSL q x p inclination
	bool   _is_M_calc;
	void   _calculate_phics();

	// tij
	double _bet;
	double _xcs; // xcs = (sqrt(rcs)-1/sqrt(rcs))*sqrt(2)/3;
	double _ycs; // ycs = ((1-sqrt(rcs))/(0.5+sqrt(rcs)))/sqrt(2);
	double _mus; // mus = (xcs^bet+ycs*xcs^(bet-1))^(1/bet);
	bool   _is_mus_calc;

	// Subloading
	double _c1; // sub-tij c
	double _c2; // sub-cam c

	// Others
	double _d1; // t1e = d1 * t1; sub-tij
	double _d2; // p1e = d2 * p1; sub-cam
	
}; // Parameters

}; // namespace MechSys

#endif
