/*************************************************************************************
 * MechSys - A C++ library to simulate (Continuum) Mechanical Systems                *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of MechSys.                                                     *
 *                                                                                   *
 * MechSys is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * MechSys is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * MechSys; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

#include <iostream>
#include <cmath>
#include "base/mechsys.h"
#include "parameters/parameters.h"
#include "models/nlelastic.h"
#include "models/subcam.h"
#include "models/subtij.h"
#include "lawinteg/feuler.h"
#include "lawinteg/meuler.h"

using namespace std;
using MechSys::Parameters;
using MechSys::Tensr2;
using MechSys::IntVars;
using MechSys::NlElastic;
using MechSys::GenericEP;
using MechSys::SubCam;
using MechSys::SubTij;
using MechSys::SchForwardEuler;
using MechSys::SchModifiedEuler;

int main(int nargs, char **args, char **envs)
{

	// Parameters
	Parameters prms;
	prms.Setlam (0.0891);
	prms.Setkap (0.0196);
	prms.Setnu  (0.2);
	prms.Setrcs (3.5);
	prms.Setenc (0.830);
	prms.Setpnc (1.0);
	prms.Setbet (1.5);
	prms.Setc1  (500.0);
	prms.Setc2  (1000.0);
	prms.Setd1  (2.0);
	prms.Setd2  (2.0);
	prms.CalculateM();
	prms.CalculateMus();

	// Initial stress
	Tensr2 Sig_0;
	Sig_0[0]=2.0;
	Sig_0[1]=2.01;
	Sig_0[2]=2.001;
	prms.Calculatevri(Sig_0);
	cout << prms << endl;

	// Increments
	Tensr2 DSig;
	Tensr2 DEps;
	DEps[0]=0.001;            DEps[1]=0.001;            DEps[2]=0.001;
	DEps[3]=0.0001*sqrt(2.0); //DEps[4]=0.0001*sqrt(2.0); DEps[5]=0.0001*sqrt(2.0);
	DEps = DEps / 1;
	
	// ---------------------------------------- Non-linear Elastic model
	
	/*
	// Model
	NlElastic model1;
	model1.Set(&prms,Sig_0);
	cout << model1 << endl;

	// Integration schemes
	SchForwardEuler <NlElastic> FE1(10);
	SchModifiedEuler<NlElastic> ME1(1e-3);

	// Stress Update
	model1.SetScheme(&FE1);
	if (!model1.StressUpdate(DEps,DSig))
		cout << "ERROR: NlElastic::StressUpdate not completed !!!\n";
	cout << "DSig = \n" << DSig << endl;
	*/

	// --------------------------------------------- Elastoplastic model

	// SubCam
	SubCam model2(MechSys::Mod);
	if (!model2.Set(&prms,Sig_0))
	{
		model2.SetSimpleOut(false);
		cout << model2 << endl;
		cout << "error: subcam model2 could not be set!\n";
		exit(1);
	}
	model2.SetSimpleOut(true);
	cout << model2 << endl;
	// SubTij
	SubTij model3;
	if (!model3.Set(&prms,Sig_0))
	{
		model3.SetSimpleOut(false);
		cout << model3 << endl;
		cout << "error: subtij model3 could not be set!\n";
		exit(1);
	}
	model3.SetSimpleOut(true);
	cout << model3 << endl;

	// Integration schemes
	SchForwardEuler <GenericEP> FE2(100);
	SchModifiedEuler<GenericEP> ME2(1e-3);

	// Stress Update
	IntVars z_ini(1,1);
	IntVars z_fin(1,1);


	cout << "/////////////////////////////////////////////////////////////////////////////// model2\n";


	// FE
	cout << "------------------------------------------ FE ----------------------------------------\n";
	model2.Reset(Sig_0);
	Tensr2  DSig_FE;
	IntVars Dz_FE(1,1);
	model2.GetIntVars(z_ini);
	model2.SetScheme(&FE2);
	if (!model2.StressUpdate(DEps,DSig_FE))
		cout << "ERROR: GenericEP::StressUpdate not completed using FE!!!\n";
	model2.GetIntVars(z_fin);
	Dz_FE = z_fin - z_ini;
	cout << "  DSig = " << DSig_FE         << endl;
	cout << "    Dz = " << Dz_FE           << endl;

	// ME
	cout << "------------------------------------------ ME ----------------------------------------\n";
	model2.Reset(Sig_0);
	Tensr2  DSig_ME;
	IntVars Dz_ME(1,1);
	model2.GetIntVars(z_ini);
	model2.SetScheme(&ME2);
	if (!model2.StressUpdate(DEps,DSig_ME))
		cout << "ERROR: GenericEP::StressUpdate not completed using ME!!!\n";
	model2.GetIntVars(z_fin);
	Dz_ME = z_fin - z_ini;
	cout << "  DSig = " << DSig_ME         << endl;
	cout << "    Dz = " << Dz_ME           << endl;
	cout << "MinNss = " << ME2.GetMinNss() << endl;
	cout << "MaxNss = " << ME2.GetMaxNss() << endl;

	
	cout << "/////////////////////////////////////////////////////////////////////////////// model3\n";


	FE2.ResetStat();
	ME2.ResetStat();

	// FE
	cout << "------------------------------------------ FE ----------------------------------------\n";
	model3.Reset(Sig_0);
	model3.GetIntVars(z_ini);
	model3.SetScheme(&FE2);
	if (!model3.StressUpdate(DEps,DSig_FE))
		cout << "ERROR: GenericEP::StressUpdate not completed using FE!!!\n";
	model3.GetIntVars(z_fin);
	Dz_FE = z_fin - z_ini;
	cout << "  DSig = " << DSig_FE         << endl;
	cout << "    Dz = " << Dz_FE           << endl;

	// ME
	cout << "------------------------------------------ ME ----------------------------------------\n";
	model3.Reset(Sig_0);
	model3.GetIntVars(z_ini);
	model3.SetScheme(&ME2);
	if (!model3.StressUpdate(DEps,DSig_ME))
		cout << "ERROR: GenericEP::StressUpdate not completed using ME!!!\n";
	model3.GetIntVars(z_fin);
	Dz_ME = z_fin - z_ini;
	cout << "  DSig = " << DSig_ME         << endl;
	cout << "    Dz = " << Dz_ME           << endl;
	cout << "MinNss = " << ME2.GetMinNss() << endl;
	cout << "MaxNss = " << ME2.GetMaxNss() << endl;
	
	return 0;
}
