/*************************************************************************************
 * Tensors - A C++ library to work with tensors                                      *
 * Copyright (C) 2005 Dorival de Moraes Pedroso <dorival_pedroso em yahoo.com>       *
 *                                                                                   *
 * This file is part of Tensors.                                                     *
 *                                                                                   *
 * Tensors is free software; you can redistribute it and/or modify it under the      *
 * terms of the GNU General Public License as published by the Free Software         *
 * Foundation; either version 2 of the License, or (at your option) any later        *
 * version.                                                                          *
 *                                                                                   *
 * Tensors is distributed in the hope that it will be useful, but WITHOUT ANY        *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.          *
 *                                                                                   *
 * You should have received a copy of the GNU General Public License along with      *
 * Tensors; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, *
 * Fifth Floor, Boston, MA 02110-1301, USA                                           *
 *************************************************************************************/

#ifndef TENSORS_TENSOR2_H
#define TENSORS_TENSOR2_H

#include <iostream>
#include <iomanip>
#include <cassert>
#include <cmath>
#include "tensor1.h"

template<typename Type, int Dim>
class Tensor2
{
public:
	// Constructor
	Tensor2(){}

	// Alternative Constructor
	Tensor2 (const Type & Val);

	// Copy Constructor
	Tensor2 (const Tensor2<Type,Dim> & Rgt);

	// Methods
	void SetWid        (int CellWidth);
	int  GetWid        () const;
	void Clear         ();
	Type GetEuclidNorm () const;

	// Specific Methods
	void SetKronecker     ();
	void SetDiagonal      (Type Value);
	void GetTranspose     (Tensor2<Type,Dim> & T2)              const;
	void GetSymmetric     (Tensor2<Type,Dim> & T2)              const;
	void GetSkewSymmetric (Tensor2<Type,Dim> & T2)              const;
	void GetTensor1       (Tensor1<Type,Dim*Dim> & T1)          const;
	void GetMandel        (Tensor1<Type,Dim*Dim> & T1)          const;
	void GetMandelSym     (Tensor1<Type,( Dim*Dim+Dim)/2> & T1) const;
	void SetColumn        (int ColNum, const Tensor1<Type,Dim> & T1);

	// Operator Overloading
	void                      operator= (const Tensor2<Type,Dim> & Rgt);
	Tensor1<Type,Dim>       & operator[](int i);
	const Tensor1<Type,Dim> & operator[](int i) const;

	// Binary operators
	Tensor2<Type,Dim> operator+ (const Tensor2<Type,Dim> & Rgt) const;
	Tensor2<Type,Dim> operator- (const Tensor2<Type,Dim> & Rgt) const;
	Tensor2<Type,Dim> operator* (const Type & Rgt)              const;
	Tensor2<Type,Dim> operator/ (const Type & Rgt)              const;

protected:
	Tensor1<Type,Dim> _tensors1[Dim];
};

// Input: Raw data
template<typename Type, int Dim>
std::istream & operator>> (std::istream & input, Tensor2<Type, Dim> & T2);

// Output: Table formatted
template<typename Type, int Dim>
std::ostream & operator<< (std::ostream & output, const Tensor2<Type, Dim> & T2);


//////////////////////////////////////////////////////////////////////////////////


// Alternative Constructor
template<typename Type, int Dim>
Tensor2<Type,Dim>::Tensor2 (const Type & Val)
{
	for (int i=0; i<Dim; ++i) 
		_tensors1[i] = Val;
}

// Copy Constructor
template<typename Type, int Dim>
Tensor2<Type,Dim>::Tensor2 (const Tensor2<Type,Dim> & Rgt)
{
	for (int i=0; i<Dim; ++i)
		_tensors1[i] = Rgt._tensors1[i];
}

// Methods
template<typename Type, int Dim>
void Tensor2<Type,Dim>::SetWid(int CellWidth)
{
	for (int i=0; i<Dim; ++i)
		_tensors1[i].SetWid(CellWidth);
}

template<typename Type, int Dim>
int Tensor2<Type,Dim>::GetWid() const
{
	return _tensors1[0].GetWid(); 
}

template<typename Type, int Dim>
void Tensor2<Type,Dim>::Clear()
{
	for (int i=0; i<Dim; ++i)
		_tensors1[i].Clear();
}

template<typename Type, int Dim>
Type Tensor2<Type,Dim>::GetEuclidNorm() const
{
	Type sum = static_cast<Type>(0);
	for (int i=0; i<Dim; ++i) 
	for (int j=0; j<Dim; ++j)
		sum = sum + _tensors1[i][j] * _tensors1[i][j];
	return static_cast<Type>(sqrt(sum));
}

// Specific Methods
template<typename Type, int Dim>
void Tensor2<Type,Dim>::SetKronecker()
{
	for (int i=0; i<Dim; ++i)
	for (int j=0; j<Dim; ++j)
	{
		if (i==j) _tensors1[i][j] = static_cast<Type>(1);
		else      _tensors1[i][j] = static_cast<Type>(0);
	}
}

template<typename Type, int Dim>
void Tensor2<Type,Dim>::SetDiagonal(Type Value)
{
	for (int i=0; i<Dim; ++i)
	for (int j=0; j<Dim; ++j)
	{
		if (i==j) _tensors1[i][j] = Value;
		else      _tensors1[i][j] = static_cast<Type>(0);
	}
}

template<typename Type, int Dim>
void Tensor2<Type,Dim>::GetTranspose(Tensor2<Type,Dim> & T2) const
{
	for (int i=0; i<Dim; ++i)
	for (int j=0; j<Dim; ++j)
		T2[i][j] = _tensors1[j][i];
}

template<typename Type, int Dim>
void Tensor2<Type,Dim>::GetSymmetric(Tensor2<Type,Dim> & T2) const
{
	for (int i=0; i<Dim; ++i)
	for (int j=0; j<Dim; ++j)
		T2[i][j] = static_cast<Type>(0.5)*(_tensors1[i][j]+_tensors1[j][i]);
}

template<typename Type, int Dim>
void Tensor2<Type,Dim>::GetSkewSymmetric(Tensor2<Type,Dim> & T2) const
{
	for (int i=0; i<Dim; ++i)
	for (int j=0; j<Dim; ++j)
		T2[i][j] = static_cast<Type>(0.5)*(_tensors1[i][j]-_tensors1[j][i]);
}

template<typename Type, int Dim>
void Tensor2<Type,Dim>::GetTensor1(Tensor1<Type,Dim*Dim> & T1) const
{
	int m=0;
	// Diagonal
	for (int i=0; i<Dim; ++i)
	{
		T1[m] = _tensors1[i][i];
		m++;
	}
	// Upper triangle
	for (int k=1; k<Dim; ++k)
	for (int i=0; i<Dim-k; ++i)
	{
		T1[m] = _tensors1[i][i+k];
		m++;
	}
	// Lower triangle
	for (int k=1; k<Dim; ++k)
	for (int j=0; j<Dim-k; ++j)
	{
		T1[m] = _tensors1[j+k][j];
		m++;
	}
}

template<typename Type, int Dim>
void Tensor2<Type,Dim>::GetMandel(Tensor1<Type,Dim*Dim> & T1) const
{
	Type sq2=sqrt(static_cast<Type>(2));
	int m=0;
	// Diagonal
	for (int i=0; i<Dim; ++i)
	{
		T1[m] = _tensors1[i][i];
		m++;
	}
	// Upper triangle
	for (int k=1; k<Dim; ++k)
	for (int i=0; i<Dim-k; ++i)
	{
		T1[m] = (_tensors1[i][i+k]+_tensors1[i+k][i])/sq2;
		m++;
	}
	// Lower triangle
	for (int k=1; k<Dim; ++k)
	for (int j=0; j<Dim-k; ++j)
	{
		T1[m] = (_tensors1[j+k][j]-_tensors1[j][j+k])/sq2;
		m++;
	}
}	

template<typename Type, int Dim>
void Tensor2<Type,Dim>::GetMandelSym(Tensor1<Type,(Dim*Dim+Dim)/2> & T1) const
{
	Type sq2=sqrt(static_cast<Type>(2));
	int m=0;
	// Diagonal
	for (int i=0; i<Dim; ++i)
	{
		T1[m] = _tensors1[i][i];
		m++;
	}
	// Upper triangle
	for (int k=1; k<Dim; ++k)
	for (int i=0; i<Dim-k; ++i)
	{
		T1[m] = sq2*_tensors1[i][i+k];
		m++;
	}
}	

template<typename Type, int Dim>
void Tensor2<Type,Dim>::SetColumn(int ColNum, const Tensor1<Type,Dim> & T1)
{
	assert(ColNum>=0);
	assert(ColNum<Dim);
	for (int i=0; i<Dim; ++i)
		_tensors1[i][ColNum] = T1[i];
}

// Operator Overloading
template<typename Type, int Dim>
void Tensor2<Type,Dim>::operator= (const Tensor2<Type,Dim> & Rgt)
{
	assert (&Rgt!=this);
	for (int i=0; i<Dim; ++i) 
		_tensors1[i] = Rgt._tensors1[i];
}

template<typename Type, int Dim>
Tensor1<Type,Dim> & Tensor2<Type,Dim>::operator[] (int i)
{
	assert (i>=0 && i<Dim);
	return _tensors1[i];
}

template<typename Type, int Dim>
const Tensor1<Type,Dim> & Tensor2<Type,Dim>::operator[] (int i) const
{
	assert (i>=0 && i<Dim);
	return _tensors1[i];
}

// Binary operators
template<typename Type, int Dim>
Tensor2<Type,Dim> Tensor2<Type,Dim>::operator+ (const Tensor2<Type,Dim> & Rgt) const
{
	Tensor2<Type,Dim> res;
	for (int i=0; i<Dim; ++i) 
		res[i] = _tensors1[i] + Rgt._tensors1[i];
	return res;
}

template<typename Type, int Dim>
Tensor2<Type,Dim> Tensor2<Type,Dim>::operator- (const Tensor2<Type,Dim> & Rgt) const
{
	Tensor2<Type,Dim> res;
	for (int i=0; i<Dim; ++i) 
		res[i] = _tensors1[i] - Rgt._tensors1[i];
	return res;
}

template<typename Type, int Dim>
Tensor2<Type,Dim> Tensor2<Type,Dim>::operator* (const Type & Rgt) const
{
	Tensor2<Type,Dim> res;
	for (int i=0; i<Dim; ++i) 
		res[i] = _tensors1[i] * Rgt;
	return res;
}

template<typename Type, int Dim>
Tensor2<Type,Dim> Tensor2<Type,Dim>::operator/ (const Type & Rgt) const
{
	Tensor2<Type,Dim> res;
	for (int i=0; i<Dim; ++i) 
		res[i] = _tensors1[i] / Rgt;
	return res;
}

// Input: Raw data
template<typename Type, int Dim>
std::istream & operator>> (std::istream & input, Tensor2<Type, Dim> & T2)
{
	for (int i=0; i<Dim; ++i)
	for (int j=0; j<Dim; ++j)
		input >> T2[i][j];
	return input;
}

// Output: Table formatted
template<typename Type, int Dim>
std::ostream & operator<< (std::ostream & output, const Tensor2<Type, Dim> & T2)
{
	for (int i=0; i<Dim; ++i)
	{
		for (int j=0; j<Dim; ++j)
			output << std::setw(T2.GetWid()) << T2[i][j] << " ";
		if (i!=Dim-1) output << std::endl;
	}
	return output;
}

#endif
